<?php

declare(strict_types = 1);

namespace Drupal\paragraphs_usage\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\paragraphs\Entity\ParagraphsType;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Paragraph usage routes.
 */
class ParagraphsUsageController extends ControllerBase {

  /**
   * Entity Field Manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * ParagraphsUsageController constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManager $entity_field_manager
   *   Entity Field Manager service.
   */
  public function __construct(EntityFieldManager $entity_field_manager) {
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_field_manager = $container->get('entity_field.manager');
    return new static($entity_field_manager);
  }

  /**
   * Get Paragraphs usage.
   *
   * @param \Drupal\paragraphs\Entity\ParagraphsType $paragraphs_type
   *   Paragraphs type.
   *
   * @return array
   *   return render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getUsage(ParagraphsType $paragraphs_type): array {
    $rows = [];
    $header = [
      'label' => $this->t('Label'),
      'machine_name' => $this->t('Machine name'),
      'type' => $this->t('Type'),
    ];
    $entity_definitions = $this->entityTypeManager()->getDefinitions();
    foreach ($entity_definitions as $entity_name => $entity_definition) {
      $types = $this->entityTypeManager()
        ->getStorage($entity_name)
        ->loadMultiple();
      foreach ($types as $type) {
        if ($bundle_of = $type->getEntityType()->getBundleOf()) {
          $base_fields = $this->entityFieldManager
            ->getFieldDefinitions($bundle_of, $type->id());
          foreach ($base_fields as $field_name => $definition) {
            if ($definition->getType() === 'entity_reference_revisions') {
              if (in_array($paragraphs_type->id(), $definition->getSetting('handler_settings')['target_bundles'])) {
                $url = Url::fromRoute('entity.' . $entity_name . '.edit_form', [$entity_name => $type->id()]);
                $type_link = Link::fromTextAndUrl($type->label(), $url);
                if (!isset($rows[$type->id()])) {
                  $rows[$type->id()] = [
                    $type_link,
                    $type->id(),
                    $entity_name,
                  ];
                }
              }
            }
          }
        }
      }
    }
    if (empty($rows)) {
      return [
        '#type' => 'item',
        '#markup' => $this->t('This paragraph is not used in any content type.'),
      ];
    }
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

}
